@@ -1,6 +1,6 @@
 {-# OPTIONS -#include "Linker.h" #-}
 -----------------------------------------------------------------------------
--- $Id: InteractiveUI.hs,v 1.159.2.7 2004/10/13 14:01:37 simonmar Exp $
+-- $Id: InteractiveUI.hs,v 1.186.2.6 2005/03/08 10:05:15 simonmar Exp $
 --
 -- GHC Interactive User Interface
 --
@@ -12,37 +12,28 @@ module InteractiveUI (
 	ghciWelcomeMsg
    ) where
 
-#include "../includes/config.h"
 #include "HsVersions.h"
 
 import CompManager
-import HscTypes		( TyThing(..), HomeModInfo(hm_linkable), HomePackageTable,
-			  isObjectLinkable, GhciMode(..) )
-import HsSyn		( TyClDecl(..), ConDecl(..), Sig(..) )
-import MkIface		( ifaceTyThing )
+import HscTypes		( GhciMode(..) )
+import IfaceSyn		( IfaceDecl(..), IfaceClassOp(..), IfaceConDecls(..),
+			  IfaceConDecl(..), IfaceType,
+		   	  pprIfaceDeclHead, pprParendIfaceType,
+			  pprIfaceForAllPart, pprIfaceType )
+import FunDeps		( pprFundeps )
 import DriverFlags
 import DriverState
 import DriverUtil	( remove_spaces )
 import Linker		( showLinkerState, linkPackages )
 import Util
-import IdInfo		( GlobalIdDetails(..) )
-import Id		( isImplicitId, idName, globalIdDetails )
-import Class		( className )
-import TyCon		( tyConName, tyConClass_maybe, isPrimTyCon, DataConDetails(..) )
-import DataCon		( dataConName )
-import FieldLabel	( fieldLabelTyCon )
-import SrcLoc		( isGoodSrcLoc )
-import Module		( showModMsg, lookupModuleEnv )
-import Name		( Name, isHomePackageName, nameSrcLoc, nameOccName,
-			  NamedThing(..) )
-import OccName		( isSymOcc )
-import BasicTypes	( defaultFixity, SuccessFlag(..) )
-import Packages
+import Name		( Name, NamedThing(..) )
+import OccName		( OccName, parenSymOcc, occNameUserString )
+import BasicTypes	( StrictnessMark(..), defaultFixity, SuccessFlag(..) )
 import Outputable
-import CmdLineOpts	( DynFlag(..), DynFlags(..), getDynFlags, saveDynFlags,
-			  restoreDynFlags, dopt_unset )
+import CmdLineOpts	( DynFlags(..), DynFlag(..), dopt )
 import Panic 		hiding ( showException )
 import Config
+import SrcLoc		( SrcLoc, isGoodSrcLoc )
 
 #ifndef mingw32_HOST_OS
 import DriverUtil( handle )
@@ -61,7 +52,7 @@ import System.Console.Readline as Readline
 
 import Control.Exception as Exception
 import Data.Dynamic
-import Control.Concurrent
+-- import Control.Concurrent
 
 import Numeric
 import Data.List
@@ -69,13 +60,16 @@ import Data.Int		( Int64 )
 import System.Cmd
 import System.CPUTime
 import System.Environment
+import System.Exit	( exitWith, ExitCode(..) )
 import System.Directory
-import System.IO as IO
+import System.IO
+import System.IO.Error as IO
 import Data.Char
 import Control.Monad as Monad
 import Foreign.StablePtr	( newStablePtr )
 
 import GHC.Exts		( unsafeCoerce# )
+import GHC.IOBase	( IOErrorType(InvalidArgument) )
 
 import Data.IORef	( IORef, newIORef, readIORef, writeIORef )
 
@@ -84,10 +78,10 @@ import System.Posix.Internals ( setNonBlockingFD )
 -----------------------------------------------------------------------------
 
 ghciWelcomeMsg =
- "   ___         ___ _\n" ++
- "  / _ \\ /\\  /\\/ __(_)\n" ++
- " / /_\\// /_/ / /  | |      GHC Interactive, version " ++ cProjectVersion ++ ", for Haskell 98.\n" ++
- "/ /_\\\\/ __  / /___| |      http://www.haskell.org/ghc/\n" ++
+ "   ___         ___ _\n"++
+ "  / _ \\ /\\  /\\/ __(_)\n"++
+ " / /_\\// /_/ / /  | |      GHC Interactive, version " ++ cProjectVersion ++ ", for Haskell 98.\n"++
+ "/ /_\\\\/ __  / /___| |      http://www.haskell.org/ghc/\n"++
  "\\____/\\/ /_/\\____/|_|      Type :? for help.\n"
 
 GLOBAL_VAR(commands, builtin_commands, [(String, String -> GHCi Bool)])
@@ -107,6 +101,7 @@ builtin_commands = [
   ("set",	keepGoing setCmd),
   ("show",	keepGoing showCmd),
   ("type",	keepGoing typeOfExpr),
+  ("kind",	keepGoing kindOfType),
   ("unset",	keepGoing unsetOptions),
   ("undef",     keepGoing undefineMacro),
   ("quit",	quit)
@@ -124,43 +119,44 @@ shortHelpText = "use :? for help.\n"
 helpText =
  " Commands available from the prompt:\n" ++
  "\n" ++
- "   <stmt>		       evaluate/run <stmt>\n" ++
- "   :add <filename> ...        add module(s) to the current target set\n" ++
- "   :browse [*]<module>	       display the names defined by <module>\n" ++
- "   :cd <dir>		       change directory to <dir>\n" ++
- "   :def <cmd> <expr>          define a command :<cmd>\n" ++
- "   :help, :?		       display this list of commands\n" ++
- "   :info [<name> ...]         display information about the given names\n" ++
- "   :load <filename> ...       load module(s) and their dependents\n" ++
- "   :module [+/-] [*]<mod> ... set the context for expression evaluation\n" ++
- "   :reload		       reload the current module set\n" ++
+ "   <stmt>                      evaluate/run <stmt>\n" ++
+ "   :add <filename> ...         add module(s) to the current target set\n" ++
+ "   :browse [*]<module>         display the names defined by <module>\n" ++
+ "   :cd <dir>                   change directory to <dir>\n" ++
+ "   :def <cmd> <expr>           define a command :<cmd>\n" ++
+ "   :help, :?                   display this list of commands\n" ++
+ "   :info [<name> ...]          display information about the given names\n" ++
+ "   :load <filename> ...        load module(s) and their dependents\n" ++
+ "   :module [+/-] [*]<mod> ...  set the context for expression evaluation\n" ++
+ "   :reload                     reload the current module set\n" ++
  "\n" ++
- "   :set <option> ...	       set options\n" ++
- "   :set args <arg> ...	       set the arguments returned by System.getArgs\n" ++
- "   :set prog <progname>       set the value returned by System.getProgName\n" ++
+ "   :set <option> ...           set options\n" ++
+ "   :set args <arg> ...         set the arguments returned by System.getArgs\n" ++
+ "   :set prog <progname>        set the value returned by System.getProgName\n" ++
  "\n" ++
- "   :show modules	       show the currently loaded modules\n" ++
- "   :show bindings	       show the current bindings made at the prompt\n" ++
+ "   :show modules               show the currently loaded modules\n" ++
+ "   :show bindings              show the current bindings made at the prompt\n" ++
  "\n" ++
- "   :type <expr>	       show the type of <expr>\n" ++
- "   :undef <cmd> 	       undefine user-defined command :<cmd>\n" ++
- "   :unset <option> ...	       unset options\n" ++
- "   :quit		       exit GHCi\n" ++
- "   :!<command>		       run the shell command <command>\n" ++
+ "   :type <expr>                show the type of <expr>\n" ++
+ "   :kind <type>                show the kind of <type>\n" ++
+ "   :undef <cmd>                undefine user-defined command :<cmd>\n" ++
+ "   :unset <option> ...         unset options\n" ++
+ "   :quit                       exit GHCi\n" ++
+ "   :!<command>                 run the shell command <command>\n" ++
  "\n" ++
- " Options for `:set' and `:unset':\n" ++
+ " Options for ':set' and ':unset':\n" ++
  "\n" ++
- "    +r			revert top-level expressions after each evaluation\n" ++
- "    +s                 print timing/memory stats after each evaluation\n" ++
- "    +t			print type after evaluation\n" ++
- "    -<flags>		most GHC command line flags can also be set here\n" ++
+ "    +r            revert top-level expressions after each evaluation\n" ++
+ "    +s            print timing/memory stats after each evaluation\n" ++
+ "    +t            print type after evaluation\n" ++
+ "    -<flags>      most GHC command line flags can also be set here\n" ++
  "                         (eg. -v2, -fglasgow-exts, etc.)\n"
 
-interactiveUI :: [FilePath] -> Maybe String -> IO ()
-interactiveUI srcs maybe_expr = do
-   dflags <- getDynFlags
 
-   cmstate <- cmInit Interactive;
+interactiveUI :: DynFlags -> [FilePath] -> Maybe String -> IO ()
+interactiveUI dflags srcs maybe_expr = do
+
+   cmstate <- cmInit Interactive dflags;
 
    -- HACK! If we happen to get into an infinite loop (eg the user
    -- types 'let x=x in x' at the prompt), then the thread will block
@@ -178,14 +174,14 @@ interactiveUI srcs maybe_expr = do
    hSetBuffering stdout NoBuffering
 
 	-- Initialise buffering for the *interpreted* I/O system
-   cmstate <- initInterpBuffering cmstate dflags
+   initInterpBuffering cmstate
 
 	-- We don't want the cmd line to buffer any input that might be
 	-- intended for the program, so unbuffer stdin.
    hSetBuffering stdin NoBuffering
 
 	-- initial context is just the Prelude
-   cmstate <- cmSetContext cmstate dflags [] ["Prelude"]
+   cmstate <- cmSetContext cmstate [] ["Prelude"]
 
 #ifdef USE_READLINE
    Readline.initialize
@@ -253,7 +249,7 @@ runGHCi paths dflags maybe_expr = do
 	    interactiveLoop is_tty show_prompt
 	Just expr -> do
 	    -- just evaluate the expression we were given
-	    runCommand expr
+	    runCommandEval expr
 	    return ()
 
   -- and finally, exit
@@ -314,8 +310,14 @@ fileLoop hdl prompt = do
    when prompt (io (putStr (mkPrompt mod imports)))
    l <- io (IO.try (hGetLine hdl))
    case l of
-	Left e | isEOFError e -> return ()
-	       | otherwise    -> io (ioError e)
+	Left e | isEOFError e		   -> return ()
+	       | InvalidArgument <- etype  -> return ()
+	       | otherwise		   -> io (ioError e)
+		where etype = ioeGetErrorType e
+		-- treat InvalidArgument in the same way as EOF:
+		-- this can happen if the user closed stdin, or
+		-- perhaps did getContents which closes stdin at
+		-- EOF.
 	Right l -> 
 	  case remove_spaces l of
 	    "" -> fileLoop hdl prompt
@@ -357,6 +359,15 @@ readlineLoop = do
 runCommand :: String -> GHCi Bool
 runCommand c = ghciHandle handler (doCommand c)
 
+-- This version is for the GHC command-line option -e.  The only difference
+-- from runCommand is that it catches the ExitException exception and
+-- exits, rather than printing out the exception.
+runCommandEval c = ghciHandle handleEval (doCommand c)
+  where 
+    handleEval (ExitException code) = io (exitWith code)
+    handleEval e                    = do showException e
+				         io (exitWith (ExitFailure 1))
+
 -- This is the exception handler for exceptions generated by the
 -- user's code; it normally just prints out the exception.  The
 -- handler must be recursive, in case showing the exception causes
@@ -393,15 +404,14 @@ runStmt stmt
  | null (filter (not.isSpace) stmt) = return []
  | otherwise
  = do st <- getGHCiState
-      dflags <- io getDynFlags
-      let dflags' = dopt_unset dflags Opt_WarnUnusedBinds
+      cmstate <- getCmState
       (new_cmstate, result) <- 
 	io $ withProgName (progname st) $ withArgs (args st) $
-	cmRunStmt (cmstate st) dflags' stmt
+	     cmRunStmt cmstate stmt
       setGHCiState st{cmstate = new_cmstate}
       case result of
 	CmRunFailed      -> return []
-	CmRunException e -> showException e >> return []
+	CmRunException e -> throw e  -- this is caught by runCommand(Eval)
 	CmRunOk names    -> return names
 
 -- possibly print the type and revert CAFs after evaluating an expression
@@ -447,26 +457,26 @@ noArgs c = throwDyn (CmdLineError ("command '" ++ c ++ "' takes no arguments"))
 GLOBAL_VAR(flush_interp,       error "no flush_interp", IO ())
 GLOBAL_VAR(turn_off_buffering, error "no flush_stdout", IO ())
 
-no_buf_cmd = "IO.hSetBuffering IO.stdout IO.NoBuffering" ++
-	     " Prelude.>> IO.hSetBuffering IO.stderr IO.NoBuffering"
-flush_cmd  = "IO.hFlush IO.stdout Prelude.>> IO.hFlush IO.stderr"
+no_buf_cmd = "System.IO.hSetBuffering System.IO.stdout System.IO.NoBuffering" ++
+	     " Prelude.>> System.IO.hSetBuffering System.IO.stderr System.IO.NoBuffering"
+flush_cmd  = "System.IO.hFlush System.IO.stdout Prelude.>> System.IO.hFlush IO.stderr"
 
-initInterpBuffering :: CmState -> DynFlags -> IO CmState
-initInterpBuffering cmstate dflags
- = do (cmstate, maybe_hval) <- cmCompileExpr cmstate dflags no_buf_cmd
+initInterpBuffering :: CmState -> IO ()
+initInterpBuffering cmstate
+ = do maybe_hval <- cmCompileExpr cmstate no_buf_cmd
 	
       case maybe_hval of
 	Just hval -> writeIORef turn_off_buffering (unsafeCoerce# hval :: IO ())
 	other	  -> panic "interactiveUI:setBuffering"
 	
-      (cmstate, maybe_hval) <- cmCompileExpr cmstate dflags flush_cmd
+      maybe_hval <- cmCompileExpr cmstate flush_cmd
       case maybe_hval of
 	Just hval -> writeIORef flush_interp (unsafeCoerce# hval :: IO ())
 	_         -> panic "interactiveUI:flush"
 
       turnOffBuffering	-- Turn it off right now
 
-      return cmstate
+      return ()
 
 
 flushInterpBuffers :: GHCi ()
@@ -486,85 +496,159 @@ help :: String -> GHCi ()
 help _ = io (putStr helpText)
 
 info :: String -> GHCi ()
-info "" = throwDyn (CmdLineError "syntax: `:i <thing-you-want-info-about>'")
-info s = do
-  let names = words s
-  init_cms <- getCmState
-  dflags <- io getDynFlags
-  let 
-    infoThings cms [] = return cms
-    infoThings cms (name:names) = do
-      (cms, stuff) <- io (cmInfoThing cms dflags name)
-      io (putStrLn (showSDocForUser unqual (
-	    vcat (intersperse (text "") (map showThing stuff))))
-         )
-      infoThings cms names
+info "" = throwDyn (CmdLineError "syntax: ':i <thing-you-want-info-about>'")
+info s  = do { let names = words s
+	     ; init_cms <- getCmState
+	     ; dflags <- getDynFlags
+	     ; let exts = dopt Opt_GlasgowExts dflags
+	     ; mapM_ (infoThing exts init_cms) names }
+  where
+    infoThing exts cms name
+	= do { stuff <- io (cmGetInfo cms name)
+	     ; io (putStrLn (showSDocForUser (cmGetPrintUnqual cms) $
+     		   vcat (intersperse (text "") (map (showThing exts) stuff)))) }
 
-    unqual = cmGetPrintUnqual init_cms
+showThing :: Bool -> GetInfoResult -> SDoc
+showThing exts (wanted_str, thing, fixity, src_loc, insts) 
+    = vcat [ showWithLoc src_loc (showDecl exts want_name thing),
+	     show_fixity fixity,
+	     vcat (map show_inst insts)]
+  where
+    want_name occ = wanted_str == occNameUserString occ
 
-    showThing (ty_thing, fixity) 
-	= vcat [ text "-- " <> showTyThing ty_thing, 
-		 showFixity fixity (getName ty_thing),
-	         ppr (ifaceTyThing True{-omit prags-} ty_thing) ]
-
-    showFixity fix name
+    show_fixity fix 
 	| fix == defaultFixity = empty
-	| otherwise            = ppr fix <+> 
-				 (if isSymOcc (nameOccName name)
-				  then ppr name
-				  else char '`' <> ppr name <> char '`')
+	| otherwise            = ppr fix <+> text wanted_str
 
-    showTyThing (AClass cl)
-       = hcat [ppr cl, text " is a class", showSrcLoc (className cl)]
-    showTyThing (ADataCon dc)
-       = hcat [ppr dc, text " is a data constructor", showSrcLoc (dataConName dc)]
-    showTyThing (ATyCon ty)
-       | isPrimTyCon ty
-       = hcat [ppr ty, text " is a primitive type constructor"]
-       | otherwise
-       = hcat [ppr ty, text " is a type constructor", showSrcLoc (tyConName ty)]
-    showTyThing (AnId   id)
-       = hcat [ppr id, text " is a ", idDescr id, showSrcLoc (idName id)]
+    show_inst (inst_ty, loc)
+	= showWithLoc loc (ptext SLIT("instance") <+> ppr inst_ty)
 
-    idDescr id
-       = case globalIdDetails id of
-	    RecordSelId lbl -> text "record selector for type" <+> ppr (fieldLabelTyCon lbl)
-	    ClassOpId cls   -> text "method in class" <+> ppr cls
-       	    otherwise       -> text "variable"
+showWithLoc :: SrcLoc -> SDoc -> SDoc
+showWithLoc loc doc 
+    = hang doc 2 (char '\t' <> show_loc loc)
+		-- The tab tries to make them line up a bit
+  where
+    show_loc loc	-- The ppr function for SrcLocs is a bit wonky
+	| isGoodSrcLoc loc = comment <+> ptext SLIT("Defined at") <+> ppr loc
+	| otherwise	   = comment <+> ppr loc
+    comment = ptext SLIT("--")
 
-	-- also print out the source location for home things
-    showSrcLoc name
-	| isHomePackageName name && isGoodSrcLoc loc
-	= hsep [ text ", defined at", ppr loc ]
+
+-- Now there is rather a lot of goop just to print declarations in a
+-- civilised way with "..." for the parts we are less interested in.
+
+showDecl :: Bool -> (OccName -> Bool) -> IfaceDecl -> SDoc
+showDecl exts want_name (IfaceForeign {ifName = tc})
+  = ppr tc <+> ptext SLIT("is a foreign type")
+
+showDecl exts want_name (IfaceId {ifName = var, ifType = ty})
+  = ppr var <+> dcolon <+> showType exts ty 
+
+showDecl exts want_name (IfaceSyn {ifName = tycon, ifTyVars = tyvars, ifSynRhs = mono_ty})
+  = hang (ptext SLIT("type") <+> pprIfaceDeclHead [] tycon tyvars)
+       2 (equals <+> ppr mono_ty)
+
+showDecl exts want_name (IfaceData {ifName = tycon, 
+		     ifTyVars = tyvars, ifCons = condecls})
+  = hang (pp_nd <+> pprIfaceDeclHead context tycon tyvars)
+       2 (add_bars (ppr_trim show_con cs))
+  where
+    show_con (IfVanillaCon { ifConOcc = con_name, ifConInfix = is_infix, ifConArgTys = tys, 
+			     ifConStricts = strs, ifConFields = flds})
+	| want_name tycon || want_name con_name || any want_name flds
+	= Just (show_guts con_name is_infix tys_w_strs flds)
+	| otherwise = Nothing
+	where
+	  tys_w_strs = tys `zip` (strs ++ repeat NotMarkedStrict)
+    show_con (IfGadtCon { ifConOcc = con_name, ifConTyVars = tvs, ifConCtxt = theta, 
+			  ifConArgTys = arg_tys, ifConResTys = res_tys, ifConStricts = strs })
+	| want_name tycon || want_name con_name
+	= Just (ppr_bndr con_name <+> colon <+> pprIfaceForAllPart tvs theta pp_tau)
+	| otherwise = Nothing
+	where
+	  tys_w_strs = arg_tys `zip` (strs ++ repeat NotMarkedStrict)
+	  pp_tau = foldr add pp_res_ty tys_w_strs
+	  pp_res_ty = ppr_bndr tycon <+> hsep (map pprParendIfaceType res_tys)
+	  add bty pp_ty = ppr_bangty bty <+> arrow <+> pp_ty
+
+    show_guts con True [ty1, ty2] flds = sep [ppr_bangty ty1, ppr con, ppr_bangty ty2]
+    show_guts con _ tys []   = ppr_bndr con <+> sep (map ppr_bangty tys)
+    show_guts con _ tys flds 
+	= ppr_bndr con <+> braces (sep (punctuate comma (ppr_trim show_fld (tys `zip` flds))))
+	where
+	  show_fld (bty, fld) | want_name tycon || want_name con || want_name fld
+			      = Just (ppr_bndr fld <+> dcolon <+> ppr_bangty bty)
+			      | otherwise = Nothing
+
+    (pp_nd, context, cs) = case condecls of
+		    IfAbstractTyCon 	      -> (ptext SLIT("data"), [],   [])
+		    IfDataTyCon (Just cxt) cs -> (ptext SLIT("data"), cxt, cs)
+		    IfDataTyCon Nothing cs    -> (ptext SLIT("data"), [],  cs)
+		    IfNewTyCon c    	      -> (ptext SLIT("newtype"), [], [c])
+
+    add_bars []      = empty
+    add_bars [c]     = equals <+> c
+    add_bars (c:cs)  = equals <+> sep (c : map (char '|' <+>) cs)
+
+    ppr_bangty (ty, str) = ppr_str str <> pprParendIfaceType ty
+    ppr_str MarkedStrict    = char '!'
+    ppr_str MarkedUnboxed   = ptext SLIT("!!")
+    ppr_str NotMarkedStrict = empty
+
+showDecl exts want_name (IfaceClass {ifCtxt = context, ifName = clas, ifTyVars = tyvars, 
+		      ifFDs = fds, ifSigs = sigs})
+  = hang (ptext SLIT("class") <+> pprIfaceDeclHead context clas tyvars
+		<+> pprFundeps fds <+> opt_where)
+       2 (vcat (ppr_trim show_op sigs))
+  where
+    opt_where | null sigs = empty
+	      | otherwise = ptext SLIT("where")
+    show_op (IfaceClassOp op dm ty) 
+	| want_name clas || want_name op 
+	= Just (ppr_bndr op <+> dcolon <+> showType exts ty)
 	| otherwise
-	= empty
-	where loc = nameSrcLoc name
+	= Nothing
 
-  cms <- infoThings init_cms names
-  setCmState cms
-  return ()
+showType :: Bool -> IfaceType -> SDoc
+showType True  ty = pprIfaceType ty -- -fglasgow-exts: print with the foralls
+showType False ty = ppr ty	    -- otherwise, print without the foralls
+
+ppr_trim :: (a -> Maybe SDoc) -> [a] -> [SDoc]
+ppr_trim show xs
+  = snd (foldr go (False, []) xs)
+  where
+    go x (eliding, so_far)
+	| Just doc <- show x = (False, doc : so_far)
+	| otherwise = if eliding then (True, so_far)
+		                 else (True, ptext SLIT("...") : so_far)
+
+ppr_bndr :: OccName -> SDoc
+-- Wrap operators in ()
+ppr_bndr occ = parenSymOcc occ (ppr occ)
+
+
+-----------------------------------------------------------------------------
+-- Commands
 
 addModule :: [FilePath] -> GHCi ()
 addModule files = do
   state <- getGHCiState
-  dflags <- io (getDynFlags)
   io (revertCAFs)			-- always revert CAFs on load/add.
   files <- mapM expandPath files
   let new_targets = files ++ targets state 
-  graph <- io (cmDepAnal (cmstate state) dflags new_targets)
-  (cmstate1, ok, mods) <- io (cmLoadModules (cmstate state) dflags graph)
+  graph <- io (cmDepAnal (cmstate state) new_targets)
+  (cmstate1, ok, mods) <- io (cmLoadModules (cmstate state) graph)
   setGHCiState state{ cmstate = cmstate1, targets = new_targets }
   setContextAfterLoad mods
+  dflags <- getDynFlags
   modulesLoadedMsg ok mods dflags
 
 changeDirectory :: String -> GHCi ()
 changeDirectory dir = do
   state    <- getGHCiState
   when (targets state /= []) $
-	io $ putStr ("Warning: changing directory causes all loaded modules to be unloaded, \n" ++
-	"because the search path has changed.\n")
-  dflags   <- io getDynFlags
-  cmstate1 <- io (cmUnload (cmstate state) dflags)
+	io $ putStr "Warning: changing directory causes all loaded modules to be unloaded,\nbecause the search path has changed.\n"
+  cmstate1 <- io (cmUnload (cmstate state))
   setGHCiState state{ cmstate = cmstate1, targets = [] }
   setContextAfterLoad []
   dir <- expandPath dir
@@ -588,9 +672,7 @@ defineMacro s = do
 
   -- compile the expression
   cms <- getCmState
-  dflags <- io getDynFlags
-  (new_cmstate, maybe_hv) <- io (cmCompileExpr cms dflags new_expr)
-  setCmState new_cmstate
+  maybe_hv <- io (cmCompileExpr cms new_expr)
   case maybe_hv of
      Nothing -> return ()
      Just hv -> io (writeIORef commands --
@@ -621,43 +703,43 @@ loadModule fs = timeIt (loadModule' fs)
 loadModule' :: [FilePath] -> GHCi ()
 loadModule' files = do
   state <- getGHCiState
-  dflags <- io getDynFlags
 
   -- expand tildes
   files <- mapM expandPath files
 
   -- do the dependency anal first, so that if it fails we don't throw
   -- away the current set of modules.
-  graph <- io (cmDepAnal (cmstate state) dflags files)
+  graph <- io (cmDepAnal (cmstate state) files)
 
   -- Dependency anal ok, now unload everything
-  cmstate1 <- io (cmUnload (cmstate state) dflags)
+  cmstate1 <- io (cmUnload (cmstate state))
   setGHCiState state{ cmstate = cmstate1, targets = [] }
 
   io (revertCAFs)  -- always revert CAFs on load.
-  (cmstate2, ok, mods) <- io (cmLoadModules cmstate1 dflags graph)
+  (cmstate2, ok, mods) <- io (cmLoadModules cmstate1 graph)
   setGHCiState state{ cmstate = cmstate2, targets = files }
 
   setContextAfterLoad mods
+  dflags <- getDynFlags
   modulesLoadedMsg ok mods dflags
 
 
 reloadModule :: String -> GHCi ()
 reloadModule "" = do
   state <- getGHCiState
-  dflags <- io getDynFlags
   case targets state of
    [] -> io (putStr "no current target\n")
    paths -> do
 	-- do the dependency anal first, so that if it fails we don't throw
 	-- away the current set of modules.
-	graph <- io (cmDepAnal (cmstate state) dflags paths)
+	graph <- io (cmDepAnal (cmstate state) paths)
 
 	io (revertCAFs)		-- always revert CAFs on reload.
 	(cmstate1, ok, mods) 
-		<- io (cmLoadModules (cmstate state) dflags graph)
+		<- io (cmLoadModules (cmstate state) graph)
         setGHCiState state{ cmstate=cmstate1 }
 	setContextAfterLoad mods
+	dflags <- getDynFlags
 	modulesLoadedMsg ok mods dflags
 
 reloadModule _ = noArgs ":reload"
@@ -684,9 +766,15 @@ modulesLoadedMsg ok mods dflags =
 typeOfExpr :: String -> GHCi ()
 typeOfExpr str 
   = do cms <- getCmState
-       dflags <- io getDynFlags
-       (new_cmstate, maybe_tystr) <- io (cmTypeOfExpr cms dflags str)
-       setCmState new_cmstate
+       maybe_tystr <- io (cmTypeOfExpr cms str)
+       case maybe_tystr of
+	  Nothing    -> return ()
+	  Just tystr -> io (putStrLn tystr)
+
+kindOfType :: String -> GHCi ()
+kindOfType str 
+  = do cms <- getCmState
+       maybe_tystr <- io (cmKindOfType cms str)
        case maybe_tystr of
 	  Nothing    -> return ()
 	  Just tystr -> io (putStrLn tystr)
@@ -698,7 +786,7 @@ shellEscape :: String -> GHCi Bool
 shellEscape str = io (system str >> return False)
 
 -----------------------------------------------------------------------------
--- Browing a module's contents
+-- Browsing a module's contents
 
 browseCmd :: String -> GHCi ()
 browseCmd m = 
@@ -709,56 +797,27 @@ browseCmd m =
 
 browseModule m exports_only = do
   cms <- getCmState
-  dflags <- io getDynFlags
 
   is_interpreted <- io (cmModuleIsInterpreted cms m)
   when (not is_interpreted && not exports_only) $
 	throwDyn (CmdLineError ("module '" ++ m ++ "' is not interpreted"))
 
-  -- temporarily set the context to the module we're interested in,
+  -- Temporarily set the context to the module we're interested in,
   -- just so we can get an appropriate PrintUnqualified
   (as,bs) <- io (cmGetContext cms)
-  cms1 <- io (if exports_only then cmSetContext cms dflags [] [prel,m]
-			      else cmSetContext cms dflags [m] [])
-  cms2 <- io (cmSetContext cms1 dflags as bs)
+  cms1 <- io (if exports_only then cmSetContext cms [] [prel,m]
+			      else cmSetContext cms [m] [])
+  cms2 <- io (cmSetContext cms1 as bs)
 
-  (cms3, things) <- io (cmBrowseModule cms2 dflags m exports_only)
-
-  setCmState cms3
+  things <- io (cmBrowseModule cms2 m exports_only)
 
   let unqual = cmGetPrintUnqual cms1 -- NOTE: cms1 with the new context
 
-      things' = filter wantToSee things
-
-      wantToSee (AnId id)    = not (isImplicitId id)
-      wantToSee (ADataCon _) = False	-- They'll come via their TyCon
-      wantToSee _ 	     = True
-
-      thing_names = map getName things
-
-      thingDecl thing@(AnId id)  = ifaceTyThing True{-omit prags-} thing
-
-      thingDecl thing@(AClass c) =
-        let rn_decl = ifaceTyThing True{-omit prags-} thing in
-	case rn_decl of
-	  ClassDecl { tcdSigs = cons } -> 
-		rn_decl{ tcdSigs = filter methodIsVisible cons }
-	  other -> other
-        where
-           methodIsVisible (ClassOpSig n _ _ _) = n `elem` thing_names
-
-      thingDecl thing@(ATyCon t) =
-        let rn_decl = ifaceTyThing True{-omit prags-} thing in
-	case rn_decl of
-	  TyData { tcdCons = DataCons cons } -> 
-		rn_decl{ tcdCons = DataCons (filter conIsVisible cons) }
-	  other -> other
-        where
-	  conIsVisible (ConDecl n _ _ _ _) = n `elem` thing_names
-
+  dflags <- getDynFlags
+  let exts = dopt Opt_GlasgowExts dflags
   io (putStrLn (showSDocForUser unqual (
-   	 vcat (map (ppr . thingDecl) things')))
-   )
+   	 vcat (map (showDecl exts (const True)) things)
+      )))
 
 -----------------------------------------------------------------------------
 -- Setting the module context
@@ -777,10 +836,9 @@ setContext str
 
 newContext mods = do
   cms <- getCmState
-  dflags <- io getDynFlags
   (as,bs) <- separate cms mods [] []
   let bs' = if null as && prel `notElem` bs then prel:bs else bs
-  cms' <- io (cmSetContext cms dflags as bs')
+  cms' <- io (cmSetContext cms as bs')
   setCmState cms'
 
 separate cmstate []           as bs = return (as,bs)
@@ -795,7 +853,6 @@ prel = "Prelude"
 
 addToContext mods = do
   cms <- getCmState
-  dflags <- io getDynFlags
   (as,bs) <- io (cmGetContext cms)
 
   (as',bs') <- separate cms mods [] []
@@ -803,14 +860,13 @@ addToContext mods = do
   let as_to_add = as' \\ (as ++ bs)
       bs_to_add = bs' \\ (as ++ bs)
 
-  cms' <- io (cmSetContext cms dflags 
+  cms' <- io (cmSetContext cms
 			(as ++ as_to_add) (bs ++ bs_to_add))
   setCmState cms'
 
 
 removeFromContext mods = do
   cms <- getCmState
-  dflags <- io getDynFlags
   (as,bs) <- io (cmGetContext cms)
 
   (as_to_remove,bs_to_remove) <- separate cms mods [] []
@@ -818,7 +874,7 @@ removeFromContext mods = do
   let as' = as \\ (as_to_remove ++ bs_to_remove)
       bs' = bs \\ (as_to_remove ++ bs_to_remove)
 
-  cms' <- io (cmSetContext cms dflags as' bs')
+  cms' <- io (cmSetContext cms as' bs')
   setCmState cms'
 
 ----------------------------------------------------------------------------
@@ -863,28 +919,21 @@ setOptions wds =
       mapM_ setOpt plus_opts
 
       -- now, the GHC flags
-      pkgs_before <- io (readIORef v_ExplicitPackages)
-      leftovers   <- io (processArgs static_flags minus_opts [])
-      pkgs_after  <- io (readIORef v_ExplicitPackages)
-
-      -- update things if the users wants more packages
-      let new_packages = pkgs_after \\ pkgs_before
-      when (not (null new_packages)) $
-	 newPackages new_packages
-
-      -- don't forget about the extra command-line flags from the 
-      -- extra_ghc_opts fields in the new packages
-      new_package_details <- io (getPackageDetails new_packages)
-      let pkg_extra_opts = concatMap extra_ghc_opts new_package_details
-      pkg_extra_dyn <- io (processArgs static_flags pkg_extra_opts [])
+      leftovers <- io $ processStaticFlags minus_opts
 
       -- then, dynamic flags
-      io $ do 
-	restoreDynFlags
-        leftovers <- processArgs dynamic_flags (leftovers ++ pkg_extra_dyn) []
-	saveDynFlags
+      dflags <- getDynFlags
+      (dflags',leftovers) <- io $ processDynamicFlags leftovers dflags
+      setDynFlags dflags'
 
-        if (not (null leftovers))
+        -- update things if the users wants more packages
+{- TODO:
+        let new_packages = pkgs_after \\ pkgs_before
+        when (not (null new_packages)) $
+  	   newPackages new_packages
+-}
+
+      if (not (null leftovers))
 		then throwDyn (CmdLineError ("unrecognised flags: " ++ 
 						unwords leftovers))
 		else return ()
@@ -937,9 +986,9 @@ optToStr RevertCAFs = "r"
 
 newPackages new_pkgs = do	-- The new packages are already in v_Packages
   state    <- getGHCiState
-  dflags   <- io getDynFlags
-  cmstate1 <- io (cmUnload (cmstate state) dflags)
+  cmstate1 <- io (cmUnload (cmstate state))
   setGHCiState state{ cmstate = cmstate1, targets = [] }
+  dflags   <- getDynFlags
   io (linkPackages dflags new_pkgs)
   setContextAfterLoad []
 
@@ -953,28 +1002,17 @@ showCmd str =
 	["linker"]   -> io showLinkerState
 	_ -> throwDyn (CmdLineError "syntax:  :show [modules|bindings]")
 
-showModules = do
-  cms <- getCmState
-  let (mg, hpt) = cmGetModInfo cms
-  mapM_ (showModule hpt) mg
-
-
-showModule :: HomePackageTable -> ModSummary -> GHCi ()
-showModule hpt mod_summary
-  = case lookupModuleEnv hpt mod of
-	Nothing	      -> panic "missing linkable"
-	Just mod_info -> io (putStrLn (showModMsg obj_linkable mod locn))
-		      where
-			 obj_linkable = isObjectLinkable (hm_linkable mod_info)
-  where
-    mod = ms_mod mod_summary
-    locn = ms_location mod_summary
+showModules
+  = do	{ cms <- getCmState
+	; let show_one ms = io (putStrLn (cmShowModule cms ms))
+	; mapM_ show_one (cmGetModuleGraph cms) }
 
 showBindings = do
   cms <- getCmState
   let
 	unqual = cmGetPrintUnqual cms
-	showBinding b = putStrLn (showSDocForUser unqual (ppr (ifaceTyThing True{-omit prags-} b)))
+--	showBinding b = putStrLn (showSDocForUser unqual (ppr (ifaceTyThing b)))
+	showBinding b = putStrLn (showSDocForUser unqual (ppr (getName b)))
 
   io (mapM_ showBinding (cmGetBindings cms))
   return ()
@@ -1018,6 +1056,10 @@ setGHCiState s = GHCi $ \r -> writeIORef r s
 getCmState = getGHCiState >>= return . cmstate
 setCmState cms = do s <- getGHCiState; setGHCiState s{cmstate=cms}
 
+getDynFlags = getCmState >>= return . cmGetDFlags
+
+setDynFlags dflags = do s <- getCmState; setCmState (cmSetDFlags s dflags)
+
 isOptionSet :: GHCiOption -> GHCi Bool
 isOptionSet opt
  = do st <- getGHCiState
