@@ -1,5 +1,5 @@
 /* -----------------------------------------------------------------------------
- * $Id: HsFFI.h,v 1.14.6.2 2001/12/13 14:48:44 simonmar Exp $
+ * $Id: HsFFI.h,v 1.20 2004/08/13 13:09:13 simonmar Exp $
  *
  * (c) The GHC Team, 2000
  *
@@ -18,7 +18,8 @@ extern "C" {
 #endif
 
 /* get types from GHC's runtime system */
-#include "config.h"
+#include "ghcconfig.h"
+#include "RtsConfig.h"
 #include "StgTypes.h"
 
 /* get limits for integral types */
@@ -148,6 +149,16 @@ typedef void*			HsForeignObj;   /* DEPRECATED */
 #define HS_DOUBLE_MAX_EXP	DBL_MAX_EXP
 #define HS_DOUBLE_MAX_10_EXP	DBL_MAX_10_EXP
 
+extern void hs_init     (int *argc, char **argv[]);
+extern void hs_exit     (void);
+extern void hs_set_argv (int argc, char *argv[]);
+extern void hs_add_root (void (*init_root)(void));
+
+extern void hs_perform_gc (void);
+
+extern void hs_free_stable_ptr (HsStablePtr sp);
+extern void hs_free_fun_ptr    (HsFunPtr fp);
+
 /* -------------------------------------------------------------------------- */
 
 #ifdef __cplusplus
