@@ -1,5 +1,5 @@
 /* -----------------------------------------------------------------------------
- * $Id: TSO.h,v 1.31.2.2 2004/03/02 11:22:57 simonmar Exp $
+ * $Id: TSO.h,v 1.40 2005/01/28 12:55:53 simonmar Exp $
  *
  * (c) The GHC Team, 1998-1999
  *
@@ -10,8 +10,6 @@
 #ifndef TSO_H
 #define TSO_H
 
-#if defined(GRAN) || defined(PAR)
-
 #if DEBUG
 #define TSO_MAGIC 4321
 #endif
@@ -34,66 +32,46 @@ typedef struct {
   StgInt   localsparks;
   rtsTime  clock;
 } StgTSOStatBuf;
-#endif
 
-#if defined(PROFILING)
+/*
+ * GRAN: We distinguish between the various classes of threads in 
+ * the system.
+ */
+typedef enum {
+  AdvisoryPriority,
+  MandatoryPriority,
+  RevalPriority
+} StgThreadPriority;
+
+/*
+ * PROFILING info in a TSO
+ */
 typedef struct {
   CostCentreStack *CCCS;	/* thread's current CCS */
 } StgTSOProfInfo;
-#else /* !PROFILING */
-typedef struct {
-} StgTSOProfInfo;
-#endif /* PROFILING */
 
-#if defined(PAR)
+/*
+ * PAR info in a TSO
+ */
 typedef StgTSOStatBuf StgTSOParInfo;
-#else /* !PAR */
-typedef struct {
-} StgTSOParInfo;
-#endif /* PAR */
 
-#if defined(DIST)
+/*
+ * DIST info in a TSO
+ */
 typedef struct {
   StgThreadPriority  priority;   
   StgInt             revalTid;   /* ToDo: merge both into 1 word */
   StgInt             revalSlot;
 } StgTSODistInfo;
-#else /* !DIST */
-typedef struct {
-} StgTSODistInfo;
-#endif /* DIST */
-
-#if defined(GRAN)
-typedef StgTSOStatBuf StgTSOGranInfo;
-#else /* !GRAN */
-typedef struct {
-} StgTSOGranInfo;
-#endif /* GRAN */
-
-
-#if defined(TICKY)
-typedef struct {
-} StgTSOTickyInfo;
-#else /* !TICKY_TICKY */
-typedef struct {
-} StgTSOTickyInfo;
-#endif /* TICKY_TICKY */
-
-typedef enum {
-    tso_state_runnable,
-    tso_state_stopped
-} StgTSOState;
 
 /*
- * The what_next field of a TSO indicates how the thread is to be run. 
+ * GRAN info in a TSO
+ */
+typedef StgTSOStatBuf StgTSOGranInfo;
+
+/*
+ * There is no TICKY info in a TSO at this time.
  */
-typedef enum {
-  ThreadRunGHC,			/* return to address on top of stack */
-  ThreadInterpret,		/* interpret this thread */
-  ThreadKilled,			/* thread has died, don't run it */
-  ThreadRelocated,		/* thread has moved, link points to new locn */
-  ThreadComplete		/* thread has finished */
-} StgTSOWhatNext;
 
 /*
  * Thread IDs are 32 bits.
@@ -101,57 +79,13 @@ typedef enum {
 typedef StgWord32 StgThreadID;
 
 /*
- * This type is returned to the scheduler by a thread that has
- * stopped for one reason or another.
+ * Type returned after running a thread.  Values of this type
+ * include HeapOverflow, StackOverflow etc.  See Constants.h for the
+ * full list.
  */
+typedef unsigned int StgThreadReturnCode;
 
-typedef enum {
-  HeapOverflow,			/* might also be StackOverflow */
-  StackOverflow,
-  ThreadYielding,
-  ThreadBlocked,
-  ThreadFinished
-} StgThreadReturnCode;
-
-/*
- * We distinguish between the various classes of threads in the system.
- */
-
-typedef enum {
-  AdvisoryPriority,
-  MandatoryPriority,
-  RevalPriority
-} StgThreadPriority;
-
-/* 
- * Threads may be blocked for several reasons.  A blocked thread will
- * have the reason in the why_blocked field of the TSO, and some
- * further info (such as the closure the thread is blocked on, or the
- * file descriptor if the thread is waiting on I/O) in the block_info
- * field.
- */
-
-typedef enum {
-  NotBlocked,
-  BlockedOnMVar,
-  BlockedOnBlackHole,
-  BlockedOnException,
-  BlockedOnRead,
-  BlockedOnWrite,
-  BlockedOnDelay
-#if defined(mingw32_TARGET_OS)
-  , BlockedOnDoProc
-#endif
-#if defined(PAR)
-  , BlockedOnGA  // blocked on a remote closure represented by a Global Address
-  , BlockedOnGA_NoSend // same as above but without sending a Fetch message
-#endif
-  , BlockedOnCCall
-  , BlockedOnCCall_NoUnblockExc // same as above but don't unblock
-				// async exceptions in resumeThread()
-} StgTSOBlockReason;
-
-#if defined(mingw32_TARGET_OS)
+#if defined(mingw32_HOST_OS)
 /* results from an async I/O request + it's ID. */
 typedef struct {
   unsigned int reqID;
@@ -163,11 +97,11 @@ typedef struct {
 typedef union {
   StgClosure *closure;
   struct StgTSO_ *tso;
-  int fd;
-#if defined(mingw32_TARGET_OS)
+  StgInt fd;	// StgInt instead of int, so that it's the same size as the ptrs
+#if defined(mingw32_HOST_OS)
   StgAsyncIOResult* async_result;
 #endif
-  unsigned int target;
+  StgWord target;
 } StgTSOBlockInfo;
 
 /*
@@ -176,6 +110,14 @@ typedef union {
  * storage manager, and won't be copied during garbage collection.
  */
 
+/* 
+ * Threads may be blocked for several reasons.  A blocked thread will
+ * have the reason in the why_blocked field of the TSO, and some
+ * further info (such as the closure the thread is blocked on, or the
+ * file descriptor if the thread is waiting on I/O) in the block_info
+ * field.
+ */
+
 /* 
  * ToDo: make this structure sensible on a non-32-bit arch.
  */
@@ -183,24 +125,35 @@ typedef union {
 typedef struct StgTSO_ {
   StgHeader          header;
 
-  struct StgTSO_*    link;	     /* Links threads onto blocking queues */
-  StgMutClosure *    mut_link;	     /* TSO's are mutable of course! */
-  struct StgTSO_*    global_link;    /* Links all threads together */
+  struct StgTSO_*    link;	     // Links threads onto blocking queues */
+  StgMutClosure *    mut_link;	     // TSO's are mutable of course! */
+  struct StgTSO_*    global_link;    // Links all threads together */
   
-  StgTSOWhatNext         what_next   : 16;
-  StgTSOBlockReason      why_blocked : 16;
-  StgTSOBlockInfo        block_info;
-  struct StgTSO_*        blocked_exceptions;
-  StgThreadID            id;
-  int                    saved_errno;
+  StgWord16           what_next;  // Values defined in Constants.h
+  StgWord16           why_blocked;  // Values defined in Constants.h
+  StgTSOBlockInfo    block_info;
+  struct StgTSO_*    blocked_exceptions;
+  StgThreadID        id;
+  int                saved_errno;
   struct StgMainThread_* main;
+  struct StgTRecHeader_ *trec;       // STM transaction record 
   
-  StgTSOTickyInfo    ticky; 
-  StgTSOProfInfo     prof;
-  StgTSOParInfo      par;
-  StgTSOGranInfo     gran;
-  StgTSODistInfo     dist;
-    
+#ifdef TICKY_TICKY
+  // TICKY-specific stuff would go here.
+#endif
+#ifdef PROFILING
+   StgTSOProfInfo prof;
+#endif
+#ifdef PAR
+   StgTSOParInfo par;
+#endif
+#ifdef GRAN
+   StgTSOGranInfo gran;
+#endif
+#ifdef DIST
+   StgTSODistInfo dist;
+#endif
+
   /* The thread stack... */
   StgWord    	     stack_size;     /* stack size in *words* */
   StgWord            max_stack_size; /* maximum stack size in *words* */
@@ -231,6 +184,8 @@ typedef struct StgTSO_ {
         BlockedOnBlackHole     the BLACKHOLE_BQ     the BLACKHOLE_BQ's queue
 	
         BlockedOnMVar          the MVAR             the MVAR's queue
+
+	BlockedOnSTM           END_TSO_QUEUE        STM wait queue(s)
 	
         BlockedOnException     the TSO              TSO->blocked_exception
 
@@ -285,4 +240,16 @@ extern StgTSO dummy_tso;
 
 #define TSO_STRUCT_SIZEW (TSO_STRUCT_SIZE / sizeof(W_))
 
+
+/* this is the NIL ptr for a TSO queue (e.g. runnable queue) */
+#define END_TSO_QUEUE  ((StgTSO *)(void*)&stg_END_TSO_QUEUE_closure)
+
+#if defined(PAR) || defined(GRAN)
+/* this is the NIL ptr for a blocking queue */
+# define END_BQ_QUEUE  ((StgBlockingQueueElement *)(void*)&stg_END_TSO_QUEUE_closure)
+/* this is the NIL ptr for a blocked fetch queue (as in PendingFetches in GUM) */
+# define END_BF_QUEUE  ((StgBlockedFetch *)(void*)&stg_END_TSO_QUEUE_closure)
+#endif
+/* ToDo?: different name for end of sleeping queue ? -- HWL */
+
 #endif /* TSO_H */
