@@ -1,5 +1,4 @@
 /* -----------------------------------------------------------------------------
- * $Id: BlockAlloc.c,v 1.16.6.2 2004/09/07 08:14:59 simonmar Exp $
  *
  * (c) The GHC Team 1998-2000
  * 
@@ -44,7 +43,7 @@ void initBlockAllocator(void)
    Allocation
    -------------------------------------------------------------------------- */
 
-static inline void
+STATIC_INLINE void
 initGroup(nat n, bdescr *head)
 {
   bdescr *bd;
@@ -194,7 +193,7 @@ allocMegaGroup(nat n)
  * pointer to the newly enlarged group p.
  */
 
-static inline bdescr *
+STATIC_INLINE bdescr *
 coalesce(bdescr *p)
 {
   bdescr *bd, *q;
@@ -262,12 +261,13 @@ static void
 freeMegaGroup(bdescr *p)
 {
   nat n;
+  void *q = p;
 
-  n = p->blocks * BLOCK_SIZE / MBLOCK_SIZE + 1;
-  for (; n > 0; (W_)p += MBLOCK_SIZE, n--) {
-    initMBlock(MBLOCK_ROUND_DOWN(p));
-    initGroup(BLOCKS_PER_MBLOCK, p);
-    freeGroup(p);
+  n = ((bdescr *)q)->blocks * BLOCK_SIZE / MBLOCK_SIZE + 1;
+  for (; n > 0; q += MBLOCK_SIZE, n--) {
+    initMBlock(MBLOCK_ROUND_DOWN(q));
+    initGroup(BLOCKS_PER_MBLOCK, (bdescr *)q);
+    freeGroup((bdescr *)q);
   }
 }
 
@@ -296,7 +296,7 @@ initMBlock(void *mblock)
 
   /* Initialise the start field of each block descriptor
    */
-  for (; block <= LAST_BLOCK(mblock); bd += 1, (lnat)block += BLOCK_SIZE) {
+  for (; block <= LAST_BLOCK(mblock); bd += 1, block += BLOCK_SIZE) {
     bd->start = block;
   }
 }
@@ -325,8 +325,8 @@ checkFreeListSanity(void)
 
   for (bd = free_list; bd != NULL; bd = bd->link) {
     IF_DEBUG(block_alloc,
-	     fprintf(stderr,"group at 0x%x, length %d blocks\n", 
-		     (nat)bd->start, bd->blocks));
+	     debugBelch("group at 0x%x, length %d blocks\n", 
+			(nat)bd->start, bd->blocks));
     ASSERT(bd->blocks > 0);
     checkWellFormedGroup(bd);
     if (bd->link != NULL) {
