@@ -1,5 +1,4 @@
 /* -----------------------------------------------------------------------------
- * $Id: FrontPanel.c,v 1.8.2.2 2004/09/07 08:16:57 simonmar Exp $
  *
  * (c) The GHC Team 2000
  *
@@ -17,7 +16,6 @@
 #include "MBlock.h"
 #include "FrontPanel.h"
 #include "Storage.h"
-#include "StoragePriv.h"
 #include "Stats.h"
 #include "RtsFlags.h"
 #include "Schedule.h"
@@ -103,7 +101,7 @@ configure_event( GtkWidget *widget, GdkEventConfigure *event STG_UNUSED,
 		      widget->allocation.width,
 		      widget->allocation.height);
 
-  fprintf(stderr, "configure!\n");
+  debugBelch("configure!\n");
   updateFrontPanel();
   return TRUE;
 }
