@@ -1,5 +1,4 @@
 /* -----------------------------------------------------------------------------
- * $Id: Itimer.h,v 1.13 2003/03/29 00:27:11 sof Exp $
  *
  * (c) The GHC Team 1998-2001
  *
@@ -12,7 +11,7 @@
 extern int startTicker( nat ms, TickProc handle_tick);
 extern int stopTicker ( void );
 
-extern unsigned int getourtimeofday   ( void );
+extern nat getourtimeofday   ( void );
 #if 0
 /* unused */
 extern void block_vtalrm_signal       ( void );
