@@ -1,5 +1,4 @@
 /* -----------------------------------------------------------------------------
- * $Id: RtsStartup.c,v 1.75.2.5 2004/09/12 11:11:17 panne Exp $
  *
  * (c) The GHC Team, 1998-2002
  *
@@ -13,19 +12,19 @@
 #include "RtsUtils.h"
 #include "RtsFlags.h"  
 #include "Storage.h"    /* initStorage, exitStorage */
-#include "StablePriv.h" /* initStablePtrTable */
 #include "Schedule.h"   /* initScheduler */
 #include "Stats.h"      /* initStats */
+#include "STM.h"        /* initSTM */
 #include "Signals.h"
 #include "Timer.h"      /* startTimer, stopTimer */
 #include "Weak.h"
 #include "Ticky.h"
 #include "StgRun.h"
-#include "StgStartup.h"
 #include "Prelude.h"		/* fixupRTStoPreludeRefs */
 #include "HsFFI.h"
 #include "Linker.h"
 #include "ThreadLabels.h"
+#include "BlockAlloc.h"
 
 #if defined(RTS_GTK_FRONTPANEL)
 #include "FrontPanel.h"
@@ -50,7 +49,7 @@
 # include "LLC.h"
 #endif
 
-#if defined(mingw32_TARGET_OS)
+#if defined(mingw32_HOST_OS)
 #include "win32/AsyncIO.h"
 #endif
 
@@ -67,17 +66,54 @@
 #include <signal.h>
 #endif
 
-// Flag Structure
-struct RTS_FLAGS RtsFlags;
-
 // Count of how many outstanding hs_init()s there have been.
 static int hs_init_count = 0;
 
-#if HAVE_TERMIOS_H
 // Here we save the terminal settings on the standard file
 // descriptors, if we need to change them (eg. to support NoBuffering
 // input).
-struct termios *saved_termios[3] = {NULL,NULL,NULL};
+static void *saved_termios[3] = {NULL,NULL,NULL};
+
+void*
+__hscore_get_saved_termios(int fd)
+{
+  return (0 <= fd && fd < (int)(sizeof(saved_termios) / sizeof(*saved_termios))) ?
+    saved_termios[fd] : NULL;
+}
+
+void
+__hscore_set_saved_termios(int fd, void* ts)
+{
+  if (0 <= fd && fd < (int)(sizeof(saved_termios) / sizeof(*saved_termios))) {
+    saved_termios[fd] = ts;
+  }
+}
+
+/* -----------------------------------------------------------------------------
+   Initialise floating point unit on x86 (currently disabled. why?)
+   -------------------------------------------------------------------------- */
+
+#define X86_INIT_FPU 0
+
+#if X86_INIT_FPU
+static void
+x86_init_fpu ( void )
+{
+  __volatile unsigned short int fpu_cw;
+
+  // Grab the control word
+  __asm __volatile ("fnstcw %0" : "=m" (fpu_cw));
+
+#if 0
+  printf("fpu_cw: %x\n", fpu_cw);
+#endif
+
+  // Set bits 8-9 to 10 (64-bit precision).
+  fpu_cw = (fpu_cw & 0xfcff) | 0x0200;
+
+  // Store the new control word back
+  __asm __volatile ("fldcw %0" : : "m" (fpu_cw));
+}
 #endif
 
 /* -----------------------------------------------------------------------------
@@ -123,14 +159,13 @@ hs_init(int *argc, char **argv[])
     /* Parse the flags, separating the RTS flags from the programs args */
     if (argc != NULL && argv != NULL) {
 	setupRtsFlags(argc, *argv, &rts_argc, rts_argv);
-	prog_argc = *argc;
-	prog_argv = *argv;
+	setProgArgv(*argc,*argv);
     }
 
 #if defined(PAR)
     /* NB: this really must be done after processing the RTS flags */
     IF_PAR_DEBUG(verbose,
-                 fprintf(stderr, "==== Synchronising system (%d PEs)\n", nPEs));
+                 debugBelch("==== Synchronising system (%d PEs)\n", nPEs));
     synchroniseSystem();             // calls initParallelSystem etc
 #endif	/* PAR */
 
@@ -158,8 +193,10 @@ hs_init(int *argc, char **argv[])
     /* initialise the stable pointer table */
     initStablePtrTable();
 
+#if defined(DEBUG)
     /* initialise thread label table (tso->char*) */
     initThreadLabelTable();
+#endif
 
 #if defined(PROFILING) || defined(DEBUG)
     initProfiling1();
@@ -178,10 +215,12 @@ hs_init(int *argc, char **argv[])
     initDefaultHandlers();
 #endif
  
-#if defined(mingw32_TARGET_OS)
+#if defined(mingw32_HOST_OS)
     startupAsyncIO();
 #endif
 
+    initSTM();
+
 #ifdef RTS_GTK_FRONTPANEL
     if (RtsFlags.GcFlags.frontpanel) {
 	initFrontPanel();
@@ -192,6 +231,10 @@ hs_init(int *argc, char **argv[])
     setlocale(LC_CTYPE,"");
 #endif
 
+#if X86_INIT_FPU
+    x86_init_fpu();
+#endif
+
     /* Record initialization times */
     stat_endInit();
 }
@@ -205,31 +248,6 @@ startupHaskell(int argc, char *argv[], void (*init_root)(void))
 }
 
 
-/* -----------------------------------------------------------------------------
-   Getting/Setting the program's arguments.
-
-   These are used by System.Environment.
-   -------------------------------------------------------------------------- */
-
-void
-getProgArgv(int *argc, char **argv[])
-{
-    if (argc) { *argc = prog_argc; }
-    if (argv) { *argv = prog_argv; }
-}
-
-void
-setProgArgv(int argc, char *argv[])
-{
-   /* Usually this is done by startupHaskell, so we don't need to call this. 
-      However, sometimes Hugs wants to change the arguments which Haskell
-      getArgs >>= ... will be fed.  So you can do that by calling here
-      _after_ calling startupHaskell.
-   */
-   prog_argc = argc;
-   prog_argv = argv;
-}
-
 /* -----------------------------------------------------------------------------
    Per-module initialisation
 
@@ -275,12 +293,14 @@ hs_add_root(void (*init_root)(void))
 	barf("hs_add_root() must be called after hs_init()");
     }
 
-    init_sp = 0;
+    /* The initialisation stack grows downward, with sp pointing 
+       to the last occupied word */
+    init_sp = INIT_STACK_BLOCKS*BLOCK_SIZE_W;
     bd = allocGroup(INIT_STACK_BLOCKS);
     init_stack = (F_ *)bd->start;
-    init_stack[init_sp++] = (F_)stg_init_ret;
+    init_stack[--init_sp] = (F_)stg_init_finish;
     if (init_root != NULL) {
-	init_stack[init_sp++] = (F_)init_root;
+	init_stack[--init_sp] = (F_)init_root;
     }
     
     cap.r.rSp = (P_)(init_stack + init_sp);
@@ -314,11 +334,6 @@ hs_exit(void)
     /* start timing the shutdown */
     stat_startExit();
     
-#if !defined(GRAN)
-    /* Finalize any remaining weak pointers */
-    finalizeWeakPointersNow();
-#endif
-    
     /* stop all running tasks */
     exitScheduler();
     
@@ -327,7 +342,7 @@ hs_exit(void)
     if (!RtsFlags.GranFlags.GranSimStats.Suppressed)
 	end_gr_simulation();
 #endif
-
+    
     /* stop the ticker */
     stopTimer();
     
@@ -348,8 +363,9 @@ hs_exit(void)
 	sigaddset(&sigset, SIGTTOU);
 	sigprocmask(SIG_BLOCK, &sigset, &old_sigset);
 	for (fd = 0; fd <= 2; fd++) {
-	    if (saved_termios[fd] != NULL) {
-		tcsetattr(fd,TCSANOW,saved_termios[fd]);
+	    struct termios* ts = (struct termios*)__hscore_get_saved_termios(fd);
+	    if (ts != NULL) {
+		tcsetattr(fd,TCSANOW,ts);
 	    }
 	}
 	sigprocmask(SIG_SETMASK, &old_sigset, NULL);
@@ -397,7 +413,7 @@ hs_exit(void)
     if (RtsFlags.TickyFlags.showTickyStats) PrintTickyInfo();
 #endif
 
-#if defined(mingw32_TARGET_OS)
+#if defined(mingw32_HOST_OS)
     shutdownAsyncIO();
 #endif
 }
@@ -441,9 +457,8 @@ stg_exit(int n)
     return;
   exit_started=rtsTrue;
 
-  IF_PAR_DEBUG(verbose, fprintf(stderr,"==-- stg_exit %d on [%x]...", n, mytid));
+  IF_PAR_DEBUG(verbose, debugBelch("==-- stg_exit %d on [%x]...", n, mytid));
   shutdownParallelSystem(n);
 #endif
   exit(n);
 }
-
