@@ -1,6 +1,8 @@
 {-# OPTIONS -#include "OSWindows\Windows_C_12\cpicture_121.h" #-}
 
-module RgnCCall_12 ( HRGN
+module RgnCCall_12
+                   {-# DEPRECATED "This module has been superseded by Graphics.UI.ObjectIO" #-}
+                   ( HRGN
                    , rgn_AND, rgn_OR, rgn_XOR, rgn_DIFF, rgn_COPY
                    , winCreateRectRgn, winCreatePolygonRgn
                    , winSetRgnToRect
