@@ -1,4 +1,6 @@
-module WindowEvent (windowEvent, module IOState) where
+module WindowEvent
+	{-# DEPRECATED "This module has been superseded by Graphics.UI.ObjectIO" #-}
+	(windowEvent, module IOState) where
 
 
 --	********************************************************************************
