@@ -1,8 +1,8 @@
 #include <windows.h>
 #include "util_121.h"
 
-extern OS WinDestroyMenu (HMENU menu, OS os);
+extern void WinDestroyMenu (HMENU menu);
 
 /*	Install the cross call procedures in the gCrossCallProcedureTable of cCrossCall_121.
 */
-extern int InstallCrossCallMenus (int ios);
+extern void InstallCrossCallMenus ();
