@@ -1,4 +1,6 @@
-module ReceiverAccess ( newReceiverHandle, newReceiver2Handle ) where
+module ReceiverAccess
+	{-# DEPRECATED "This module has been superseded by Graphics.UI.ObjectIO" #-}
+	( newReceiverHandle, newReceiver2Handle ) where
 
 
 --	********************************************************************************
