@@ -1,4 +1,6 @@
-module StdBitmap where
+module StdBitmap
+	{-# DEPRECATED "This module has been superseded by Graphics.UI.ObjectIO" #-}
+	where
 
 
 --	Clean Object I/O library, version 1.2
