@@ -1,4 +1,6 @@
-module StdFileSelect(selectInputFile, selectOutputFile, selectDirectory) where
+module StdFileSelect(selectInputFile,
+	{-# DEPRECATED "This module has been superseded by Graphics.UI.ObjectIO" #-}
+	selectOutputFile, selectDirectory) where
 
 --	Clean Object I/O library, version 1.2
 
