@@ -1,4 +1,6 @@
-module StdIOBasic (module StdIOBasic) where
+module StdIOBasic
+	{-# DEPRECATED "This module has been superseded by Graphics.UI.ObjectIO" #-}
+	(module StdIOBasic) where
 
 
 --	********************************************************************************
