@@ -1,6 +1,8 @@
 {-# OPTIONS -#include "OSWindows\Windows_C_12\cpicture_121.h" #-}
 
-module StdPicture(module StdPictureDef, module StdPicture, Draw(..), doScreenDraw) where
+module StdPicture(module
+	{-# DEPRECATED "This module has been superseded by Graphics.UI.ObjectIO" #-}
+	StdPictureDef, module StdPicture, Draw(..), doScreenDraw) where
 
 --	********************************************************************************
 --	Clean to Haskell Standard Object I/O library, version 1.2
