@@ -1,4 +1,6 @@
-module StdPictureDef(module StdPictureDef, Font) where
+module StdPictureDef(module
+	{-# DEPRECATED "This module has been superseded by Graphics.UI.ObjectIO" #-}
+	StdPictureDef, Font) where
 
 
 --	********************************************************************************
