@@ -452,49 +452,49 @@ LRESULT prim_defWindowProc(HWND arg1,DWORD arg2,WPARAM arg3,LPARAM arg4)
       return((LRESULT)(res1));} while(0);
 }
 void* prim_getClientRect(HWND arg1)
-{ static struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
+{ static struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
   RECT res1;int gc_failed;
 	    char* gc_failstring;
   do { BOOL success = GetClientRect(arg1,&res1);
       if ((gc_failed = ( !success ))) {gc_failstring =  ErrorString("GetClientRect")  ;}
       else {gc_failed = 0;}
-      gc_result.gc_res2 = (LONG)((res1).left);
-      gc_result.gc_res4 = (LONG)((res1).top);
-      gc_result.gc_res6 = (LONG)((res1).right);
-      gc_result.gc_res8 = (LONG)((res1).bottom);
+      gc_result.gc_res1 = (LONG)((res1).left);
+      gc_result.gc_res2 = (LONG)((res1).top);
+      gc_result.gc_res3 = (LONG)((res1).right);
+      gc_result.gc_res4 = (LONG)((res1).bottom);
       gc_result.gc_failed = gc_failed;
       gc_result.gc_failstring = gc_failstring;
       
       return(&gc_result);} while(0);
 }
-LONG access_prim_getClientRect_gc_res2(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res2);}
-LONG access_prim_getClientRect_gc_res4(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res4);}
-LONG access_prim_getClientRect_gc_res6(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res6);}
-LONG access_prim_getClientRect_gc_res8(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res8);}
-int access_prim_getClientRect_gc_failed(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
-void* access_prim_getClientRect_gc_failstring(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
+LONG access_prim_getClientRect_gc_res1(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res1);}
+LONG access_prim_getClientRect_gc_res2(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res2);}
+LONG access_prim_getClientRect_gc_res3(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res3);}
+LONG access_prim_getClientRect_gc_res4(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res4);}
+int access_prim_getClientRect_gc_failed(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
+void* access_prim_getClientRect_gc_failstring(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
 void* prim_getWindowRect(HWND arg1)
-{ static struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
+{ static struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
   RECT res1;int gc_failed;
 	    char* gc_failstring;
   do { BOOL success = GetWindowRect(arg1,&res1);
       if ((gc_failed = ( !success ))) {gc_failstring =  ErrorString("GetWindowRect")  ;}
       else {gc_failed = 0;}
-      gc_result.gc_res2 = (LONG)((res1).left);
-      gc_result.gc_res4 = (LONG)((res1).top);
-      gc_result.gc_res6 = (LONG)((res1).right);
-      gc_result.gc_res8 = (LONG)((res1).bottom);
+      gc_result.gc_res1 = (LONG)((res1).left);
+      gc_result.gc_res2 = (LONG)((res1).top);
+      gc_result.gc_res3 = (LONG)((res1).right);
+      gc_result.gc_res4 = (LONG)((res1).bottom);
       gc_result.gc_failed = gc_failed;
       gc_result.gc_failstring = gc_failstring;
       
       return(&gc_result);} while(0);
 }
-LONG access_prim_getWindowRect_gc_res2(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res2);}
-LONG access_prim_getWindowRect_gc_res4(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res4);}
-LONG access_prim_getWindowRect_gc_res6(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res6);}
-LONG access_prim_getWindowRect_gc_res8(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res8);}
-int access_prim_getWindowRect_gc_failed(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
-void* access_prim_getWindowRect_gc_failstring(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
+LONG access_prim_getWindowRect_gc_res1(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res1);}
+LONG access_prim_getWindowRect_gc_res2(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res2);}
+LONG access_prim_getWindowRect_gc_res3(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res3);}
+LONG access_prim_getWindowRect_gc_res4(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res4);}
+int access_prim_getWindowRect_gc_failed(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
+void* access_prim_getWindowRect_gc_failstring(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
 void* prim_invalidateRect(HWND arg1,LPRECT arg2,int arg3)
 { static struct {HsInt gc_failed;HsPtr gc_failstring;} gc_result;
   int gc_failed;
@@ -509,42 +509,44 @@ void* prim_invalidateRect(HWND arg1,LPRECT arg2,int arg3)
 }
 int access_prim_invalidateRect_gc_failed(void *ptr){ return(((struct {HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
 void* access_prim_invalidateRect_gc_failstring(void *ptr){ return(((struct {HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
-void* prim_screenToClient(HWND arg1,LONG gc_arg3,LONG gc_arg5)
-{ static struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
+void* prim_screenToClient(HWND arg1,LONG gc_arg2,LONG gc_arg3)
+{ static struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
   int gc_failed;
   char* gc_failstring;POINT scr;
+  (scr).x = (LONG)gc_arg2; (scr).y = (LONG)gc_arg3;
   do { BOOL success = ScreenToClient(arg1, &scr);
       if ((gc_failed = (  !success  ))) {gc_failstring =  ErrorString("ScreenToClient")  ;}
       else {gc_failed = 0;}
-      gc_result.gc_res2 = (LONG)((scr).x);
-      gc_result.gc_res4 = (LONG)((scr).y);
+      gc_result.gc_res1 = (LONG)((scr).x);
+      gc_result.gc_res2 = (LONG)((scr).y);
       gc_result.gc_failed = gc_failed;
       gc_result.gc_failstring = gc_failstring;
       
       return(&gc_result);} while(0);
 }
-LONG access_prim_screenToClient_gc_res2(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res2);}
-LONG access_prim_screenToClient_gc_res4(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res4);}
-int access_prim_screenToClient_gc_failed(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
-void* access_prim_screenToClient_gc_failstring(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
-void* prim_clientToScreen(HWND hwnd,LONG gc_arg3,LONG gc_arg5)
-{ static struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
+LONG access_prim_screenToClient_gc_res1(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res1);}
+LONG access_prim_screenToClient_gc_res2(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res2);}
+int access_prim_screenToClient_gc_failed(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
+void* access_prim_screenToClient_gc_failstring(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
+void* prim_clientToScreen(HWND hwnd,LONG gc_arg2,LONG gc_arg3)
+{ static struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
   int gc_failed;
   char* gc_failstring;POINT cli;
+  (cli).x = (LONG)gc_arg2; (cli).y = (LONG)gc_arg3;
   do { BOOL success = ClientToScreen(hwnd,&cli);
       if ((gc_failed = (  !success  ))) {gc_failstring =  ErrorString("ClientToScreen")  ;}
       else {gc_failed = 0;}
-      gc_result.gc_res2 = (LONG)((cli).x);
-      gc_result.gc_res4 = (LONG)((cli).y);
+      gc_result.gc_res1 = (LONG)((cli).x);
+      gc_result.gc_res2 = (LONG)((cli).y);
       gc_result.gc_failed = gc_failed;
       gc_result.gc_failstring = gc_failstring;
       
       return(&gc_result);} while(0);
 }
-LONG access_prim_clientToScreen_gc_res2(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res2);}
-LONG access_prim_clientToScreen_gc_res4(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res4);}
-int access_prim_clientToScreen_gc_failed(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
-void* access_prim_clientToScreen_gc_failstring(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
+LONG access_prim_clientToScreen_gc_res1(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res1);}
+LONG access_prim_clientToScreen_gc_res2(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res2);}
+int access_prim_clientToScreen_gc_failed(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
+void* access_prim_clientToScreen_gc_failstring(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
 void* prim_setWindowText(HWND arg1,char * arg2)
 { static struct {HsInt gc_failed;HsPtr gc_failstring;} gc_result;
   int gc_failed;
@@ -655,50 +657,52 @@ int prim_showWindow(HWND arg1,DWORD arg2)
       
       return((int)(res1));} while(0);
 }
-void* prim_adjustWindowRect(LONG gc_arg3,LONG gc_arg5,LONG gc_arg7,LONG gc_arg9,DWORD arg2,int arg3)
-{ static struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
+void* prim_adjustWindowRect(LONG gc_arg2,LONG gc_arg3,LONG gc_arg4,LONG gc_arg5,DWORD arg2,int arg3)
+{ static struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
   int gc_failed;
   char* gc_failstring;RECT arg1;
+  (arg1).left = (LONG)gc_arg2; (arg1).top = (LONG)gc_arg3; (arg1).right = (LONG)gc_arg4; (arg1).bottom = (LONG)gc_arg5;
   do { BOOL success = AdjustWindowRect(&arg1, arg2, arg3);
       if ((gc_failed = (  !success  ))) {gc_failstring =  ErrorString("AdjustWindowRect")  ;}
       else {gc_failed = 0;}
-      gc_result.gc_res2 = (LONG)((arg1).left);
-      gc_result.gc_res4 = (LONG)((arg1).top);
-      gc_result.gc_res6 = (LONG)((arg1).right);
-      gc_result.gc_res8 = (LONG)((arg1).bottom);
+      gc_result.gc_res1 = (LONG)((arg1).left);
+      gc_result.gc_res2 = (LONG)((arg1).top);
+      gc_result.gc_res3 = (LONG)((arg1).right);
+      gc_result.gc_res4 = (LONG)((arg1).bottom);
       gc_result.gc_failed = gc_failed;
       gc_result.gc_failstring = gc_failstring;
       
       return(&gc_result);} while(0);
 }
-LONG access_prim_adjustWindowRect_gc_res2(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res2);}
-LONG access_prim_adjustWindowRect_gc_res4(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res4);}
-LONG access_prim_adjustWindowRect_gc_res6(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res6);}
-LONG access_prim_adjustWindowRect_gc_res8(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res8);}
-int access_prim_adjustWindowRect_gc_failed(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
-void* access_prim_adjustWindowRect_gc_failstring(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
-void* prim_adjustWindowRectEx(LONG gc_arg3,LONG gc_arg5,LONG gc_arg7,LONG gc_arg9,DWORD arg2,int arg3,DWORD arg4)
-{ static struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
+LONG access_prim_adjustWindowRect_gc_res1(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res1);}
+LONG access_prim_adjustWindowRect_gc_res2(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res2);}
+LONG access_prim_adjustWindowRect_gc_res3(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res3);}
+LONG access_prim_adjustWindowRect_gc_res4(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res4);}
+int access_prim_adjustWindowRect_gc_failed(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
+void* access_prim_adjustWindowRect_gc_failstring(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
+void* prim_adjustWindowRectEx(LONG gc_arg2,LONG gc_arg3,LONG gc_arg4,LONG gc_arg5,DWORD arg2,int arg3,DWORD arg4)
+{ static struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
   int gc_failed;
   char* gc_failstring;RECT arg1;
+  (arg1).left = (LONG)gc_arg2; (arg1).top = (LONG)gc_arg3; (arg1).right = (LONG)gc_arg4; (arg1).bottom = (LONG)gc_arg5;
   do { BOOL success = AdjustWindowRectEx(&arg1, arg2, arg3, arg4);
       if ((gc_failed = (  !success  ))) {gc_failstring =  ErrorString("AdjustWindowRectEx")  ;}
       else {gc_failed = 0;}
-      gc_result.gc_res2 = (LONG)((arg1).left);
-      gc_result.gc_res4 = (LONG)((arg1).top);
-      gc_result.gc_res6 = (LONG)((arg1).right);
-      gc_result.gc_res8 = (LONG)((arg1).bottom);
+      gc_result.gc_res1 = (LONG)((arg1).left);
+      gc_result.gc_res2 = (LONG)((arg1).top);
+      gc_result.gc_res3 = (LONG)((arg1).right);
+      gc_result.gc_res4 = (LONG)((arg1).bottom);
       gc_result.gc_failed = gc_failed;
       gc_result.gc_failstring = gc_failstring;
       
       return(&gc_result);} while(0);
 }
-LONG access_prim_adjustWindowRectEx_gc_res2(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res2);}
-LONG access_prim_adjustWindowRectEx_gc_res4(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res4);}
-LONG access_prim_adjustWindowRectEx_gc_res6(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res6);}
-LONG access_prim_adjustWindowRectEx_gc_res8(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res8);}
-int access_prim_adjustWindowRectEx_gc_failed(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
-void* access_prim_adjustWindowRectEx_gc_failstring(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
+LONG access_prim_adjustWindowRectEx_gc_res1(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res1);}
+LONG access_prim_adjustWindowRectEx_gc_res2(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res2);}
+LONG access_prim_adjustWindowRectEx_gc_res3(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res3);}
+LONG access_prim_adjustWindowRectEx_gc_res4(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res4);}
+int access_prim_adjustWindowRectEx_gc_failed(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
+void* access_prim_adjustWindowRectEx_gc_failstring(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
 int prim_anyPopup()
 { int res1;
   do {res1 = AnyPopup();
@@ -749,10 +753,11 @@ void* prim_bringWindowToTop(HWND arg1)
 }
 int access_prim_bringWindowToTop_gc_failed(void *ptr){ return(((struct {HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
 void* access_prim_bringWindowToTop_gc_failstring(void *ptr){ return(((struct {HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
-void* prim_childWindowFromPoint(HWND hwnd,LONG gc_arg3,LONG gc_arg5)
+void* prim_childWindowFromPoint(HWND hwnd,LONG gc_arg2,LONG gc_arg3)
 { static struct {HsPtr res1;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
   HWND res1;int gc_failed;
 	    char* gc_failstring;POINT p;
+  (p).x = (LONG)gc_arg2; (p).y = (LONG)gc_arg3;
   do { BOOL success = ChildWindowFromPoint(hwnd,p);
       if ((gc_failed = (  !success  ))) {gc_failstring =  ErrorString("ChildWindowFromPoint")  ;}
       else {gc_failed = 0;}
@@ -765,10 +770,11 @@ void* prim_childWindowFromPoint(HWND hwnd,LONG gc_arg3,LONG gc_arg5)
 HWND access_prim_childWindowFromPoint_res1(void *ptr){ return(((struct {HsPtr res1;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->res1);}
 int access_prim_childWindowFromPoint_gc_failed(void *ptr){ return(((struct {HsPtr res1;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
 void* access_prim_childWindowFromPoint_gc_failstring(void *ptr){ return(((struct {HsPtr res1;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
-void* prim_childWindowFromPointEx(HWND hwnd,LONG gc_arg3,LONG gc_arg5,DWORD arg2)
+void* prim_childWindowFromPointEx(HWND hwnd,LONG gc_arg2,LONG gc_arg3,DWORD arg2)
 { static struct {HsPtr res1;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
   HWND res1;int gc_failed;
 	    char* gc_failstring;POINT p;
+  (p).x = (LONG)gc_arg2; (p).y = (LONG)gc_arg3;
   do { BOOL success = ChildWindowFromPointEx(hwnd,p,arg2);
       if ((gc_failed = (  !success  ))) {gc_failstring =  ErrorString("ChildWindowFromPointEx")  ;}
       else {gc_failed = 0;}
@@ -1104,23 +1110,23 @@ void* prim_releaseDC(HWND arg1,HDC arg2)
 int access_prim_releaseDC_gc_failed(void *ptr){ return(((struct {HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
 void* access_prim_releaseDC_gc_failstring(void *ptr){ return(((struct {HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
 void* prim_getDCOrgEx(HDC arg1)
-{ static struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
+{ static struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
   POINT res1;int gc_failed;
 	     char* gc_failstring;
   do { BOOL success = GetDCOrgEx(arg1,&res1);
       if ((gc_failed = ( !success ))) {gc_failstring = ErrorString("GetDCOrgEx") ;}
       else {gc_failed = 0;}
-      gc_result.gc_res2 = (LONG)((res1).x);
-      gc_result.gc_res4 = (LONG)((res1).y);
+      gc_result.gc_res1 = (LONG)((res1).x);
+      gc_result.gc_res2 = (LONG)((res1).y);
       gc_result.gc_failed = gc_failed;
       gc_result.gc_failstring = gc_failstring;
       
       return(&gc_result);} while(0);
 }
-LONG access_prim_getDCOrgEx_gc_res2(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res2);}
-LONG access_prim_getDCOrgEx_gc_res4(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res4);}
-int access_prim_getDCOrgEx_gc_failed(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
-void* access_prim_getDCOrgEx_gc_failstring(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
+LONG access_prim_getDCOrgEx_gc_res1(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res1);}
+LONG access_prim_getDCOrgEx_gc_res2(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res2);}
+int access_prim_getDCOrgEx_gc_failed(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
+void* access_prim_getDCOrgEx_gc_failstring(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
 void* prim_hideCaret(HWND arg1)
 { static struct {HsInt gc_failed;HsPtr gc_failstring;} gc_result;
   int gc_failed;
@@ -1178,27 +1184,28 @@ void* prim_destroyCaret()
 int access_prim_destroyCaret_gc_failed(void *ptr){ return(((struct {HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
 void* access_prim_destroyCaret_gc_failstring(void *ptr){ return(((struct {HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
 void* prim_getCaretPos()
-{ static struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
+{ static struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
   POINT res1;int gc_failed;
 	     char* gc_failstring;
   do {  BOOL success = GetCaretPos(&res1);
       if ((gc_failed = ( !success ))) {gc_failstring = ErrorWin("GetCaretPos") ;}
       else {gc_failed = 0;}
-      gc_result.gc_res2 = (LONG)((res1).x);
-      gc_result.gc_res4 = (LONG)((res1).y);
+      gc_result.gc_res1 = (LONG)((res1).x);
+      gc_result.gc_res2 = (LONG)((res1).y);
       gc_result.gc_failed = gc_failed;
       gc_result.gc_failstring = gc_failstring;
       
       return(&gc_result);} while(0);
 }
-LONG access_prim_getCaretPos_gc_res2(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res2);}
-LONG access_prim_getCaretPos_gc_res4(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res4);}
-int access_prim_getCaretPos_gc_failed(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
-void* access_prim_getCaretPos_gc_failstring(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
-void* prim_setCaretPos(LONG gc_arg3,LONG gc_arg5)
+LONG access_prim_getCaretPos_gc_res1(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res1);}
+LONG access_prim_getCaretPos_gc_res2(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res2);}
+int access_prim_getCaretPos_gc_failed(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
+void* access_prim_getCaretPos_gc_failstring(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
+void* prim_setCaretPos(LONG gc_arg2,LONG gc_arg3)
 { static struct {HsInt gc_failed;HsPtr gc_failstring;} gc_result;
   int gc_failed;
   char* gc_failstring;POINT arg1;
+  (arg1).x = (LONG)gc_arg2; (arg1).y = (LONG)gc_arg3;
   do { BOOL success = SetCaretPos(arg1.x, arg1.y);
       if ((gc_failed = ( !success ))) {gc_failstring = ErrorWin("SetCaretPos") ;}
       else {gc_failed = 0;}
